# TODO: https://github.com/changhsinlee/pytest-mock-examples/blob/master/tests/test_mocking_constant_mocker.py


# import asyncio
# from omega_tx.mock import Hygrometer
# import sys
#
#
# async def read():
#     async with Hygrometer() as tx:
#         sys.stdout.write(str(await tx.get()))
#
# asyncio.run(read())


# import asyncio
# from omega_tx import Barometer
#
#
# async def read_once():
#     async with Barometer('192.168.1.171') as tx:
#         print(await tx.get())
#
# asyncio.run(read_once())
